<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\API\User\AdminUpdateRequest;
use App\Http\Requests\API\User\BatchCreateRequest;
use App\Http\Requests\API\User\CreateRequest;
use App\Http\Requests\API\User\UpdateRequest;
use App\Http\Resources\API\UserInfoResource;
use App\Models\Enums\RequirementStatus;
use App\Models\Role;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;

class UserController extends Controller
{
    public function info()
    {
        $user = Auth::user();

        return new UserInfoResource($user);
    }

    public function assignRole(Request $request, string $id)
    {
        $user = User::query()->findOrFail($id);

        $roleName = $request->get('role_name');
        if ($roleName) {
            $role = Role::query()->where("guard_name", "api")->where('name', $roleName)->firstOrFail();
            $user->role_id = $role->id;
            $user->save();
        }

        $roles = $roleName ? [$roleName] : [];

        $user->syncRoles($roles);

        return $this->noContent();
    }

    public function index(){
        $user = User::all();
        return UserInfoResource::collection($user);
    }

    /**
     * add a new User
     * @return \Illuminate\Http\Response
     */
    public function store(CreateRequest $request){
        $password = Hash::make($request->password);
        $user=$request->all();
        $user['password']=$password;
        User::create([
            ...$user,
            'created_by' => Auth::id(),
        ]);
//        TODO:发送邮箱给目标用户
        return $this->created();
    }


    /**
     * batchCreate  User,为ditto时参考上一条
     * @return \Illuminate\Http\Response
     */
    public function batchStore(BatchCreateRequest $request){
        $userData = $request->all();
        DB::transaction(function () use ($userData) {
        foreach ($userData as $k => $data) {
            $user = new User();
            if ($k != 0) {
                $userData[$k]["department_id"] = $userData[$k]["department_id"] == 'ditto' ? $userData[$k - 1]["department_id"] : $userData[$k]["department_id"];
                $userData[$k]["role_id"] = $userData[$k]["role_id"] == 'ditto' ? $userData[$k - 1]["role_id"] : $userData[$k]["role_id"];
            }
            $userData[$k]['password'] = Hash::make($userData[$k]['password']);
            $user->fill([
                ...$userData[$k],
                'created_by' => Auth::id(),
            ]);
            //        TODO:发送邮箱给目标用户
            $user->save();
        }
        });
//        TODO:发送邮箱给目标用户
        return $this->created();
    }

    /**
     * enable or ban users 启用或禁用用户
     * @param Request $request
     * @return \Illuminate\Http\Response
     */
    public function status(Request $request,string $status){
        //只能删除自己公司的;超管除外
        if (Auth::user()->super_admin){
            User::whereIn('id', $request->user_id)->update(['status' => $status]);
        }
        else{
            User::whereIn('id', $request->user_id)->where('company_id',Auth::user()->company_id)->update(['status' => $status]);
        }
        return $this->created();
    }

    public function destroy(string $id)
    {

    }

    public function show(string $id)
    {

    }

    public function update(Request $request,string $id)
    {
        dd($id);
    }
    /**
     * @param CreateRequest $request
     * @return \Illuminate\Http\Response
     * 修改个人信息
     */
    public function updateInfo(UpdateRequest $request)
    {
        $user = User::findOrFail(Auth::user()->id);
        $user->fill([
            ...$request->except(['username','role_id','department_id','company_id'])
        ]);
        $user->save();
        return $this->noContent();
    }

    /**
     * @param AdminUpdateRequest $request 修改主体
     * @param string $id 用户id
     * @return \Illuminate\Http\Response
     * 超管修改用户的信息
     */
    public function updateUserInfo(AdminUpdateRequest $request, string $id)
    {
        $user = User::findOrFail($id);
        $user->fill([
            ...$request->all()
        ]);
        $user->save();
        return $this->noContent();

    }

}
